package com.clx.performance.component;

import cn.hutool.core.collection.CollectionUtil;
import com.clx.order.feign.OrderFeign;
import com.clx.order.param.feign.UpdateOrderInfoParam;
import com.clx.order.vo.feign.FeignOrderVO;
import com.clx.performance.constant.RedisConstants;
import com.clx.performance.dao.OrderGoodsDao;
import com.clx.performance.enums.OrderGoodsStatusEnum;
import com.clx.performance.model.OrderGoods;
import com.msl.common.base.Optional;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.Date;
import java.util.List;
import java.util.Set;

@Component
@Slf4j
public class OrderGoodsStatusLazyComponent implements InitializingBean {

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private OrderGoodsDao orderGoodsDao;

    @Autowired
    private OrderFeign orderFeign;

    public void expireProduce(LocalDateTime localDateTime, String orderGoodsNo) {
        stringRedisTemplate.opsForZSet().add(RedisConstants.ORDER_GOODS_STATUS_LAZY, orderGoodsNo, localDateTime.toInstant(ZoneOffset.of("+8")).toEpochMilli());
    }


    public void consuming() {
        long nowTimeMillis = System.currentTimeMillis();
        LocalDateTime nowDateTime = new Date(nowTimeMillis).toInstant().atOffset(ZoneOffset.of("+8")).toLocalDateTime();
        Set<ZSetOperations.TypedTuple<String>> orderGoodsIds = stringRedisTemplate.opsForZSet().rangeByScoreWithScores(
                RedisConstants.ORDER_GOODS_STATUS_LAZY,
                0, nowTimeMillis  //延时任务score最小值
                //延时任务score最大值（当前时间）
        );
        if (!CollectionUtil.isEmpty(orderGoodsIds)) {
            for (ZSetOperations.TypedTuple<String> orderGoodsId : orderGoodsIds) {
                log.info("货单" + orderGoodsId + "过了最晚拉运时间");
                String orderGoodsNo = orderGoodsId.getValue();
                OrderGoods orderGoods = orderGoodsDao.getByOrderGoodsNo(orderGoodsNo).get();
                FeignOrderVO orderInfoFeign = orderFeign.getOrderInfoFeign(orderGoods.getOrderNo());

                if (!OrderGoodsStatusEnum.Status.COMPLETED.getCode().equals(orderInfoFeign.getOrderStatus()) && OrderGoodsStatusEnum.Status.SUCCESS.getCode().equals(orderGoods.getOrderGoodsStatus())) {
                    //如果当前货单已完成，则判断之前所有货单是否等于订单总吨数，等于则更新订单为已完成
                    List<OrderGoods> list = orderGoodsDao.getOrderGoodsListByOrderNoAndLastArriveSendTime(orderGoods.getOrderNo(), nowDateTime);
                    BigDecimal childSum = list.stream().map(OrderGoods::getExtractWeight).reduce(BigDecimal.ZERO, BigDecimal::add);
                    if (childSum.compareTo(new BigDecimal(orderInfoFeign.getTransportWeight())) == 0) {
                        //补充更新订单状态 已完成
                        UpdateOrderInfoParam updateOrderInfoParam = new UpdateOrderInfoParam();
                        updateOrderInfoParam.setOrderId(orderInfoFeign.getId());
                        updateOrderInfoParam.setOrderStatus(OrderGoodsStatusEnum.Status.SUCCESS.getCode());
                        orderFeign.updateOrderInfo(updateOrderInfoParam);
                    }
                } else {
                    //更新货单已完结，并返还货单吨数
                    orderGoodsDao.updateOrderGoodsStatusByOrderGoodsNo(orderGoodsNo, OrderGoodsStatusEnum.Status.COMPLETED.getCode());
                    BigDecimal residueTransportWeight = orderGoods.getResidueTransportWeight();
                    UpdateOrderInfoParam updateOrderInfoParam = new UpdateOrderInfoParam();
                    updateOrderInfoParam.setOrderId(orderInfoFeign.getId());
                    updateOrderInfoParam.setResidueWeight(residueTransportWeight);
                    //返还订单剩余吨数
                    orderFeign.updateOrderInfo(updateOrderInfoParam);
                }

                stringRedisTemplate.opsForZSet().remove(RedisConstants.ORDER_GOODS_STATUS_LAZY, orderGoodsId.getValue());
            }
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        Optional<List<OrderGoods>> optional = orderGoodsDao.getOrderGoodsList();
        if (optional.isPresent()) {
            for (OrderGoods orderGoods : optional.get()) {
                stringRedisTemplate.opsForZSet().add(RedisConstants.ORDER_GOODS_STATUS_LAZY, orderGoods.getOrderGoodsNo(), orderGoods.getLastArriveSendTime().toInstant(ZoneOffset.of("+8")).toEpochMilli());
            }
        }
    }
}
